==> SyFi-0.1.0.tar.gz: SyFi-0.1/sandbox/vec.h <==
#ifndef VEC_IS_INCLUDED
#define VEC_IS_INCLUDED

#include <vector>
#include <string>
#include <ginac/basic.h>
#include <ginac/ex.h>

namespace GiNaC {


/** Helper template to allow initialization of matrices via an overloaded
 *  comma operator (idea stolen from Blitz++). */
template <typename T, typename It>
class vec_init {
public:
	vec_init(It i) : iter(i) {}

	vec_init<T, It> operator,(const T & x)
	{
		*iter = x;
		return vec_init<T, It>(++iter);
	}

	// The following specializations produce much tighter code than the
	// general case above

	vec_init<T, It> operator,(int x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

	vec_init<T, It> operator,(unsigned int x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

	vec_init<T, It> operator,(long x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

	vec_init<T, It> operator,(unsigned long x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

	vec_init<T, It> operator,(double x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

	vec_init<T, It> operator,(const symbol & x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

private:
	vec_init();
	It iter;
};


/** Symbolic matrices. */
class vec : public basic
{
	GINAC_DECLARE_REGISTERED_CLASS(vec, basic)
	
	// other constructors
public:
	vec(unsigned r);
	vec(const exvector & m2);
	vec(const lst & l);

	// First step of initialization of vec with a comma-separated seqeuence
	// of expressions. Subsequent steps are handled by vec_init<>::operator,().
	vec_init<ex, exvector::iterator> operator=(const ex & x)
	{
		m[0] = x;
		return vec_init<ex, exvector::iterator>(++m.begin());
	}
	
	// functions overriding virtual functions from base classes
public:
	size_t nops() const;
	ex op(size_t i) const;
	ex & let_op(size_t i);
	ex eval(int level=0) const;
	ex evalm() const {return *this;}
	ex subs(const exmap & m, unsigned options = 0) const;
	ex eval_indexed(const basic & i) const;
	ex add_indexed(const ex & self, const ex & other) const;
	ex scalar_mul_indexed(const ex & self, const numeric & other) const;
	bool contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const;

protected:
	bool match_same_type(const basic & other) const;
	unsigned return_type() const { return return_types::noncommutative; };
	
	// non-virtual functions in this class
public:
	unsigned size () const        /// Get number of rows.
		{ return size; }
	vec add(const vec & other) const;
	vec sub(const vec & other) const;
	vec mul(const vec & other) const;
	vec mul(const numeric & other) const;
	vec mul_scalar(const ex & other) const;
	const ex & operator() (unsigned ro) const;
	ex & operator() (unsigned ro);
	vec & set(unsigned ro,const ex & value) { (*this)(ro) = value; return *this; }
protected:

	void print_elements(const print_context & c, const char *row_start, const char *row_end, const char *row_sep, const char *col_sep) const;
	void do_print(const print_context & c, unsigned level) const;
	void do_print_latex(const print_latex & c, unsigned level) const;
	void do_print_python_repr(const print_python_repr & c, unsigned level) const;
	
// member variables
protected:
	unsigned size;             ///< number of rows
	exvector m;               ///< representation (cols indexed first)
};


// wrapper functions around member functions

inline size_t nops(const vec & m)
{ return m.nops(); }

inline ex expand(const vec & m, unsigned options = 0)
{ return m.expand(options); }

inline ex eval(const vec & m, int level = 0)
{ return m.eval(level); }

inline ex evalf(const vec & m, int level = 0)
{ return m.evalf(level); }

inline unsigned size (const vec & m)
{ return m.size; }



// utility functions

/** Specialization of is_exactly_a<vec>(obj) for vec objects. */
template<> inline bool is_exactly_a<vec>(const basic & obj)
{
	return obj.tinfo()==TINFO_vec;
}

/** Convert list of lists to vec. */
extern ex lst_to_vec(const lst & l);

/** Create an  unit vec. */
extern ex unit_vec(unsigned r);

}

#endif VEC_IS_INCLUDED

==> SyFi-0.3.2.tar.gz: SyFi-0.3.2/sandbox/vec.h <==
#ifndef VEC_IS_INCLUDED
#define VEC_IS_INCLUDED

#include <vector>
#include <string>
#include <ginac/basic.h>
#include <ginac/ex.h>

const unsigned TINFO_vec        = 0x111e0001U;

namespace GiNaC {


/** Helper template to allow initialization of matrices via an overloaded
 *  comma operator (idea stolen from Blitz++). */
template <typename T, typename It>
class vec_init {
public:
	vec_init(It i) : iter(i) {}

	vec_init<T, It> operator,(const T & x)
	{
		*iter = x;
		return vec_init<T, It>(++iter);
	}

	// The following specializations produce much tighter code than the
	// general case above

	vec_init<T, It> operator,(int x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

	vec_init<T, It> operator,(unsigned int x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

	vec_init<T, It> operator,(long x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

	vec_init<T, It> operator,(unsigned long x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

	vec_init<T, It> operator,(double x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

	vec_init<T, It> operator,(const symbol & x)
	{
		*iter = T(x);
		return vec_init<T, It>(++iter);
	}

private:
	vec_init();
	It iter;
};


/** Symbolic matrices. */
class vec : public basic
{
	GINAC_DECLARE_REGISTERED_CLASS(vec, basic)
	
	// other constructors
public:
	vec(unsigned r);
	vec(const exvector & m2);
	vec(const lst & l);

	// First step of initialization of vec with a comma-separated seqeuence
	// of expressions. Subsequent steps are handled by vec_init<>::operator,().
	vec_init<ex, exvector::iterator> operator=(const ex & x)
	{
		m[0] = x;
		return vec_init<ex, exvector::iterator>(++m.begin());
	}
	
	// functions overriding virtual functions from base classes
public:
	size_t nops() const;
	ex op(size_t i) const;
	ex & let_op(size_t i);
	ex eval(int level=0) const;
	ex evalm() const {return *this;}
	ex subs(const exmap & m, unsigned options = 0) const;
	ex eval_indexed(const basic & i) const;
	ex add_indexed(const ex & self, const ex & other) const;
	ex scalar_mul_indexed(const ex & self, const numeric & other) const;
	bool contract_with(exvector::iterator self, exvector::iterator other, exvector & v) const;

protected:
	bool match_same_type(const basic & other) const;
	unsigned return_type() const { return return_types::noncommutative; };
	
	// non-virtual functions in this class
public:
	unsigned size () const  { return size_; }
	vec add(const vec & other) const;
	vec sub(const vec & other) const;
	vec mul(const vec & other) const;
	vec mul(const numeric & other) const;
	vec mul_scalar(const ex & other) const;
	const ex & operator() (unsigned ro) const;
	ex & operator() (unsigned ro);
	vec & set(unsigned ro,const ex & value) { (*this)(ro) = value; return *this; }
protected:

	void print_elements(const print_context & c, const char *row_start, const char *row_end, const char *row_sep) const;
	void do_print(const print_context & c, unsigned level) const;
	void do_print_latex(const print_latex & c, unsigned level) const;
	void do_print_python_repr(const print_python_repr & c, unsigned level) const;
	
// member variables
protected:
	unsigned size_;             ///< number of rows
	exvector m;               ///< representation (cols indexed first)
};


// wrapper functions around member functions

inline size_t nops(const vec & m)
{ return m.nops(); }

inline ex expand(const vec & m, unsigned options = 0)
{ return m.expand(options); }

inline ex eval(const vec & m, int level = 0)
{ return m.eval(level); }

inline ex evalf(const vec & m, int level = 0)
{ return m.evalf(level); }

inline unsigned size (const vec & m)
{ return m.size(); }



// utility functions

/** Specialization of is_exactly_a<vec>(obj) for vec objects. */
template<> inline bool is_exactly_a<vec>(const basic & obj)
{
	return obj.tinfo()==TINFO_vec;
}

/** Convert list of lists to vec. */
extern ex lst_to_vec(const lst & l);

/** Create an  unit vec. */
extern ex unit_vec(unsigned r);

}

#endif
